#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   fore_rain_data_convert.py
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/10/11 13:14     douly      1.0         fore_rain_data
"""
import sys
import time

import toml
import argparse
from datetime import datetime, timedelta
# import lib
from time import sleep
from threading import Thread

import pyodbc

from sqlalchemy import Column, Integer, Float, String, Text, BigInteger, DateTime, create_engine, func, and_
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from com.tzx.tools.clock_tool import to_standard_hour, time_from_str
from com.tzx.tools.database_tool import get_session

class GribWeatherFore(model_base):
	__tablename__ = "tb_gribwf"
	id = Column("id", Integer, nullable=False)
	filename = Column("filename", String(512), primary_key=True)
	wfsrc = Column("wfsrc", String(16))
	area = Column("area", String(16))
	facname = Column("facname", String(32))
	wfdatetime = Column("wfdatetime", BigInteger, primary_key=True)
	wfinterval = Column("wfinterval", Integer)
	wfhours = Column("wfhours", Integer)
	wfhour = Column("wfhour", Integer, primary_key=True)
	upperleft = Column("upperleft", String(50), nullable=False)
	lowerright = Column("lowerright", String(50), nullable=False)
	width = Column("width", Integer, nullable=False)
	height = Column("height", Integer, nullable=False)
	stepx = Column("stepx", Float, nullable=False)
	stepy = Column("stepy", Float, nullable=False)
	data = Column("data", Text, nullable=False)
	data1 = Column("data1", Text, nullable=False)
	status = Column("status", Integer, nullable=False)
	score = Column("score", Text, nullable=False)
	scoretime = Column("scoretime", String(50), nullable=False)
	score1 = Column("score1", Text, nullable=False)
	score2 = Column("score2", Text, nullable=False)
	datatype = Column("datatype", Integer, nullable=False)
	data3 = Column("data3", Text, nullable=False)


class ForeRainPoint(model_base):  # 原始数据表
	__tablename__ = 'ForeRainPoint_B'
	PID = Column("PID", Integer, primary_key=True, autoincrement=True)
	LGTD = Column("LGTD", Float)
	LTTD = Column("LTTD", Float)
	DataSourceID = Column("DataSourceID", Integer)


class ForeRainData(model_base):
	__tablename__ = "ForeRainData_M_B"
	TID = Column("TID", Integer, primary_key=True, autoincrement=True)
	PID = Column("PID", Integer, nullable=False)
	TM = Column("TM", DateTime, nullable=False)
	DRP = Column("DRP", Float)


class Points:
	def __init__(self, lgtd, lttd):
		self.lgtd = lgtd
		self.lttd = lttd

	def __hash__(self):
		return hash(str(self.lgtd) + str(self.lttd))

	def __eq__(self, other):
		if self.lttd == other.lttd and self.lgtd == other.lgtd:
			return True
		return False


fore_point_dict = dict()


def add_hunan_point(model_str=None):
	point_list = list()
	for i in range(0, 13456):
		lgtd = (i % 116) * 0.05 + 108.65
		lttd = 30.25 - (i // 116) * 0.05
		tmp_record = ForeRainPoint(LGTD=lgtd, LTTD=lttd, DataSourceID=4301)
		point_list.append(tmp_record)
	if not model_str:
		model_str = 'mssql+pyodbc://sa:3edc9ijn~@192.168.0.151/TZX_RES_MODEL_SHYJ?driver=sql+server'
	engine = create_engine(model_str, echo=True)
	DBSession = sessionmaker(bind=engine)
	session = DBSession()
	session.add_all(point_list)
	session.commit()
	session.close()


def load_fore_point(model_conn_str):
	session = get_session(model_conn_str)
	rets = session.query(ForeRainPoint).filter(ForeRainPoint.DataSourceID == 21).all()
	for r in rets:
		p = Points(round(r.LGTD, 2), round(r.LTTD, 2))
		fore_point_dict[p] = r.PID
	session.close()


def task_thread(base_time: datetime, wfhour: int, data_str: str, model_conn_str):
	target_time = base_time + timedelta(hours=wfhour)
	assert len(data_str) > 2
	data_str.strip('')
	data_str = data_str[1:-1]  # 去除头尾的 {}
	ceils = data_str.split(',')
	assert len(ceils) == 116 * 116
	fore_data = list()
	for i in range(0, 116 * 116):
		lgtd = (i % 116) * 0.05 + 108.65
		lttd = 30.25 - (i // 116) * 0.05
		pid = fore_point_dict[Points(round(lgtd, 2), round(lttd, 2))]
		fore_data.append(ForeRainData(PID=pid, TM=target_time, DRP=float(ceils[i])))
	model_session = get_session(model_conn_str)
	model_session.add_all(fore_data)
	model_session.commit()
	fore_data.clear()
	model_session.close()


def proc_task(base_time, model_conn_str, fore_conn_str):
	wfdatetime = '{:0>4d}{:0>2d}{:0>2d}{:0>2d}00'.format(base_time.year, base_time.month, base_time.day, base_time.hour)
	model_session = get_session(model_conn_str)
	fore_session = get_session(fore_conn_str)
	print(wfdatetime)
	# if wfdatetime is not None: #or model_time < wfdatetime:  # 目标数据库中没有改时间的数据
	results = fore_session.query(GribWeatherFore).filter(and_(GribWeatherFore.wfdatetime == wfdatetime, GribWeatherFore.wfhour <=12)).all()
	if len(results) == 0:
		print("Time : {0} not exist".format(wfdatetime))
	thread_list = list()
	for r in results:
		t = Thread(target=task_thread, args=(base_time, r.wfhour, r.data, model_conn_str,))
		thread_list.append(t)
		t.start()
		# task_thread(base_time, r.wfhour, r.data, model_conn_str)

	is_finish = False
	while not is_finish:
		is_finish = True
		for t in thread_list:
			if t.isAlive():
				is_finish = False
				break
		sleep(2)
	fore_session.close()

	print(1)


if __name__ == '__main__':
	# add_hunan_point()
	parser = argparse.ArgumentParser(description='配置文件路径')
	parser.add_argument('path', type=str, help='配置文件路径')
	args = parser.parse_args()
	config = toml.load(args.path)
	model_str = config.get("fore").get("model")
	#'mssql+pyodbc://sa:3edc9ijn~@192.168.0.151/TZX_RES_MODEL_SHYJ?driver=sql+server'
	fore_str = config.get("fore").get("forerwdb")
	start_time_str = config.get("fore").get("start_time")
	#'mssql+pyodbc://sa:123456@.\\dly/hunanfore?driver=sql+server'
	load_fore_point(model_str)
	start_time = time_from_str(start_time_str)
	while model_str is not None and fore_str is not None:
		thread_list2 = list()
		# 202110110800
		temp_time = datetime(year=2021, month=10, day=11, hour=20)
		while start_time < temp_time:
			# t1 = Thread(target=proc_task, args = [start_time, model_str, fore_str])
			proc_task(start_time, model_str, fore_str)
			# thread_list2.append(t1)
			# t1.start()
			start_time = start_time + timedelta(hours=12)

		# is_all_done = False
		# while not is_all_done:
		# 	is_all_done = True
		# 	for t1 in thread_list2:
		# 		if t1.isAlive():
		# 			is_all_done =False
		# 			break
		# 	sleep(3)

	print('Exiting..')
	time.sleep(5)
	sys.exit(0)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   web_tool.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/11/15 11:16     douly      1.0         web_tool
"""
# import lib
from flask import Response


def response_headers(content: str):
	"""


	:param content:
	:return:
	"""
	resp = Response(content)
	resp.headers['Access-Control-Allow-Origin'] = '*'
	return resp


def response_headers_json(content: str):
	"""
	返回json的response, 完成跨域

	:param content: json str
	:return:
	"""
	resp = Response(content)
	resp.content_type = 'application/json'
	resp.headers['Access-Control-Allow-Method'] = '*'
	resp.headers['Access-Control-Allow-Headers'] = '*'
	resp.headers['Access-Control-Allow-Origin'] = '*'
	return resp


def response_headers_png(content):
	resp = Response(content)
	resp.content_type = 'image/png'
	resp.headers['Access-Control-Allow-Origin'] = '*'
	return resp



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   image_tool.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021-12-10 11:45     douly      1.0         image_tool
"""
# import lib
import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont


def draw_chinese(_image, text, positive, font_size=20, font_color=(0, 0, 0, 255)):
	"""
	在图像上

	:param _image:
	:param text:
	:param positive:
	:param font_size:
	:param font_color:
	:return:
	"""
	cv2img = cv2.cvtColor(_image, cv2.COLOR_BGR2RGBA)  # cv2和PIL中颜色的hex码的储存顺序不同
	pilimg = Image.fromarray(cv2img)
	# PIL图片上打印汉字
	draw = ImageDraw.Draw(pilimg)  # 图片上打印
	font = ImageFont.truetype("SIMLI.TTF", font_size, encoding="utf-8")  # 参数1：字体文件路径，参数2：字体大小
	draw.text(positive, text, font_color, font=font)  # 参数1：打印坐标，参数2：文本，参数3：字体颜色，参数4：字体格式
	char_img = cv2.cvtColor(np.array(pilimg), cv2.COLOR_RGB2BGRA)  # PIL图片转cv2 图片

	return char_img


def fill_ceil(src, threshold=0):
	"""
	一种填充灰度图像孔洞的方法, 值threshold的点 则取附近3*3网格内,6个最大数的平均值,可以扩展添加算子和

	:param src:
	:param threshold:
	:return:
	"""
	for _r in range(2, src.shape[0] - 2):
		for _c in range(2, src.shape[1] - 2):
			if src[_r, _c] > threshold:
				continue
			else:
				_arr = src[_r - 1: _r + 1, _c - 1:_c + 1]
				_arr = _arr.flatten()
				# _arr.sort(reverse=True)
				_arr = -np.sort(-_arr)
				src[_r, _c] = np.average(_arr[:6])

	return src


def channel_3_4(src):
	(rols, cols) = src.shape[:2]
	dst = np.zeros((rols, cols, 4), dtype=np.uint8)
	for r in range(src.shape[0]):
		for c in range(src.shape[1]):
			_val = list(src[r, c])
			_val.append(255)
			dst[r, c] = _val
	return dst


def img_float32(img):
	return img.copy() if img.dtype != 'uint8' else (img / 255.).astype('float32')


def over(fore_ground, background):
	fore_ground, background = img_float32(fore_ground), img_float32(background)
	(fb, fg, fr, fa), (bb, bg, br, ba) = cv2.split(fore_ground), cv2.split(background)
	color_fg, color_bg = cv2.merge((fb, fg, fr)), cv2.merge((bb, bg, br))
	alpha_fg, alpha_bg = np.expand_dims(fa, axis=-1), np.expand_dims(ba, axis=-1)

	color_fg[fa == 0] = [0, 0, 0]
	color_bg[ba == 0] = [0, 0, 0]

	a = fa + ba * (1 - fa)
	a[a == 0] = np.NaN
	color_over = (color_fg * alpha_fg + color_bg * alpha_bg * (1 - alpha_fg)) / np.expand_dims(a, axis=-1)
	color_over = np.clip(color_over, 0, 1)
	color_over[a == 0] = [0, 0, 0]

	result_float32 = np.append(color_over, np.expand_dims(a, axis=-1), axis=-1)
	return (result_float32 * 255).astype('uint8')


def overlay_with_transparency(background, fore_ground, xmin=0, ymin=0, trans_percent=1):
	"""
	将前景透明图像叠加到后景图像上

	:param background: a 4 channel image, use as background
	:param fore_ground: a 4 channel image, use as foreground
	:param xmin: a coordinate in background. from where the fore_ground will be put
	:param ymin: a coordinate in background. from where the fore_ground will be put
	:param trans_percent: transparency of fore_ground. [0.0,1.0]
	:return: a merged image
	"""
	# we assume all the input image has 4 channels
	assert (background.shape[-1] == 4 and fore_ground.shape[-1] == 4)
	fore_ground = fore_ground.copy()
	roi = background[ymin:ymin + fore_ground.shape[0], xmin:xmin + fore_ground.shape[1]].copy()

	# opencv 通道拆分
	b, g, r, a = cv2.split(fore_ground)

	# opencv 通道合并
	fore_ground = cv2.merge((b, g, r, (a * trans_percent).astype(fore_ground.dtype)))

	roi_over = over(fore_ground, roi)
	# 注意用的都是copy,不会影响到原图
	result = background.copy()
	result[ymin:ymin + fore_ground.shape[0], xmin:xmin + fore_ground.shape[1]] = roi_over
	return result


def merge_image_by_image(left_image, right_image):
	"""
	将两张图片并列放置并且合并, 左右

	:param left_image:
	:param right_image:
	:return:
	"""
	(lrows, lcols, lchannels) = left_image.shape
	(rrows, rcols, rchannels) = right_image.shape
	assert lrows == rrows and lcols == rcols and lchannels == rchannels
	dst_image = np.zeros((lrows, lcols * 2, lchannels))
	dst_image[0:lrows, 0:lcols] = left_image
	dst_image[0: lrows, lcols:2 * lcols] = right_image
	return dst_image.copy()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   update.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/10/18 16:30     douly      1.0         update
"""
# import lib
import sys
import pyodbc
from datetime import datetime, timedelta
from operator import and_

from sqlalchemy import create_engine, Column, String, DateTime, Float

from tzx_easy_py.tools.database_tool import model_base, get_session


class HuNanOraclePptn(model_base):  # 原始数据表
	__tablename__ = 'ST_PPTN_R'
	STCD = Column('STCD', String(50), primary_key=True, nullable=False)
	TM = Column('TM', DateTime, primary_key=True, nullable=False)
	DRP = Column('DRP', Float)
	INTV = Column('INTV', Float)
	PDR = Column('PDR', Float)
	DYP = Column('DYP', Float)
	WTH = Column('WTH', String(50))


delta = timedelta(days=5)
start_time = datetime(year=2021, month=1, day=1, hour=0, minute=0)

now = datetime.now()


while start_time < now:
	end_time = start_time + delta
	# oracle_session = get_session('oracle://rwdb_hn:rwdb_hn@192.168.0.71:1521/orcl')
	oracle_session = get_session('oracle://hyits:hyits@10.43.15.4:1521/hnfxorcl')
	result = oracle_session.query(HuNanOraclePptn).filter(and_(HuNanOraclePptn.TM > start_time, HuNanOraclePptn.TM <= end_time)).all()
	print(len(result))

	if len(result) <= 0:
		# print('Finish.', start_time)
		continue

	insert_data = list()
	for r in result:
		insert_data.append(HuNanOraclePptn(STCD=r.STCD, TM=r.TM, DRP=r.DRP, INTV=r.INTV, PDR=r.PDR, DYP=r.DYP, WTH=r.WTH))

	mssql_session = get_session('mssql+pyodbc://sa:3EDC9ijn~@10.43.1.206/hyits?driver=sql+server')
	# mssql_session = get_session('mssql+pyodbc://sa:123456@.\\dly/foredata?driver=sql+server')
	mssql_session.add_all(insert_data)
	mssql_session.commit()
	oracle_session.close()
	mssql_session.close()
	print(start_time)
	start_time = end_time

print('End')
sys.exit(0)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   clock_server.py    
@Contact :   mitiandameng@gmail.tzx
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/11/19 12:38     douly      1.0         clock_server
"""
# import lib



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   database_tool.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/11/10 9:42     douly      1.0         database_tool
"""
# import lib
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker


from sqlalchemy.ext.declarative import declarative_base
model_base = declarative_base()


def get_session(orm_str: str):
	"""
	获取SQLAlchemy的会话对象

	:param orm_str: SQLAlchemy的数据库连接串
	:return:
	"""
	src_engine = create_engine(orm_str, encoding='utf-8')
	# 创建DBSession类型:
	SourceSession = sessionmaker(bind=src_engine)
	return SourceSession()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   system_tool.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/11/21 12:26     douly      1.0         system_tool
"""
# import lib


class Points:
	def __init__(self, lgtd, lttd):
		self.lgtd = lgtd
		self.lttd = lttd

	def __hash__(self):
		return hash(str(self.lgtd) + str(self.lttd))

	def __eq__(self, other):
		if self.lttd == other.lttd and self.lgtd == other.lgtd:
			return True
		return False


def is_number(s):
	"""
	判断给定的字符串能否被转为浮点数

	:param s:
	:return:
	"""
	try:
		float(s)
		return True
	except ValueError:
		pass

	try:
		import unicodedata
		unicodedata.numeric(s)
		return True
	except (TypeError, ValueError):
		pass

	return False



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   clock_tool.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/11/5 9:37     douly      1.0         clock_tool
"""
# import lib
import json
import re
from datetime import datetime, timedelta

import redis


def to_standard_hour(tm: datetime, hs: list):
	"""
	传入的时间转为标准数据时间

	:param tm: 时间
	:param hs: 标准hour , 由大到小排列
	:return:
	"""
	assert len(hs)
	hs.sort(reverse=True)
	h = tm.hour
	for th in hs:
		if h > th:
			return datetime(year=tm.year, month=tm.month, day=tm.day, hour=th)

	return datetime(year=tm.year, month=tm.month, day=tm.day, hour=hs[0]) - timedelta(days=1)


REFMT1 = r"(\d{4}-\d{1,2}-\d{1,2})"
REFMT2 = r"(\d{4}-\d{1,2}-\d{1,2}\s\d{1,2})"
REFMT3 = r"(\d{4}-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2})"
REFMT4 = r"(\d{4}-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2})"

DFT1 = '%Y-%m-%d'
DFT2 = '%Y-%m-%d %H'
DFT3 = '%Y-%m-%d %H:%M'
DFT4 = '%Y-%m-%d %H:%M:%S'
DFT6 = '%Y%m%d%H%M'
DFT7 = '%m/%d %H'
DFT21 = '%Y%m%d%H'
DFT22 = '%Y年%m月%d日%H时'

DFT51 = '%H时%M分'


def time_from_str(time_str: str):
	"""
	从字符串转化为datetime对象

	:param time_str:
	:return:
	"""
	try:
		mat = re.findall(REFMT4, time_str)
		if len(mat) > 0:
			return datetime.strptime(time_str, DFT4)
		mat = re.findall(REFMT3, time_str)
		if len(mat) > 0:
			return datetime.strptime(time_str, DFT3)
		mat = re.findall(REFMT2, time_str)
		if len(mat) > 0:
			return datetime.strptime(time_str, DFT2)
		mat = re.findall(REFMT1, time_str)
		if len(mat) > 0:
			return datetime.strptime(time_str, DFT1)
	except Exception as e:
		print(e)

	return None


def update_time_from_redis(redis_client: redis.Redis, topic: str, method):
	"""
	更新时间监视

	:param redis_client: redis 客户端
	:param topic: 订阅主题
	:param method: 执行方法
	:return:
	"""
	# rc = redis.StrictRedis(host=addr, port=port, password=redis_pwd)
	while True:
		ps = redis_client.pubsub()
		ps.subscribe(topic)
		for item in ps.listen():  # 监听状态：有消息发布了就拿过来
			if item['type'] == 'message':
				time = json.loads(str(item['data'], encoding='utf-8'))['end_time']
				print(time)
				ftime = time_from_str(time)
				try:
					method(ftime)
				except Exception as e:
					print(e)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   soil_water_tools.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021-12-16 13:29     douly      1.0         soil_water_tools
"""


# import lib


def calc_soil_data(pptn_data: dict, wms, kas, last_soil, reduce=0.6):
	"""
	计算含水量数据

	:param pptn_data: 降雨数据 {point:{TM, DRP}} ,TM需要整理为整点数据
	:param wms: 持水量
	:param kas: 折减系数
	:param last_soil:上个时刻的含水率
	:param reduce: 没有上个时刻含水率时的初值折减
	:return:
	"""
	data_begin_tm = max(pptn_data.keys())

	for _p, _wm in wms.items():
		_ka = kas[_p]
		_last_val = 0.6 * _wm
		if _p in last_soil:
			_last_val = last_soil[_p]
	#
	for _p, records in pptn_data.items():
		_tmp_record = last_soil.copy()  # 含水百分比 %
		_wm = wms[_p]
		for _tm, _drp in records.items():
			_last_soil = reduce * _wm
			if _p in _tmp_record:
				_last_soil = _tmp_record[_p] / 100.0 * _wm

			_this_soil_w = min(kas[_p] * (_last_soil + _drp), _wm)
			swp = _this_soil_w / _wm
			_tmp_record[_p] = int(swp * 100)  # 含水率



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py.py    
@Contact :   mitiandameng@gmail.tzx
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021/11/19 12:01     douly      1.0         __init__.py
"""
# import lib



import setuptools

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="tzx_py_tools",
    version="0.0.2",
    description="A small example package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    ],
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   easy_type.py    
@Contact :   mitiandameng@gmail.com
@License :   (C)Copyright 2020-2021, TianZhiXiang.
@Software:   PyCharm
@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2021-12-10 18:17     douly      1.0         easy_type
"""
# import lib
from pydantic import BaseModel


class GeoRect(BaseModel):
	left: float
	right: float
	top: float
	bottom: float


class GeoPoint(BaseModel):
	lgtd: float
	lttd: float


class ScreenRect(BaseModel):
	width: int
	height: int


class ScreenPoint(BaseModel):
	x: int
	y: int

